from collections import defaultdict


PACKAGES = {
    'setuptools': [
        '21.2.1',  # pypy3
        '28.8.0',  # jython python34 python35 pypy pypy3
        '36.8.0',  # virtualenv (py26)
        '39.0.1',  # python36 python37
        '39.1.0',  # virtualenv
    ],
    'pip': [
        '8.1.2',  # pypy3
        '9.0.1',  # jython python34 python35 pypy pypy3
        '9.0.3',  # virtualenv (py26)
        '10.0.1',  # virtualenv python36 python37
    ],
    'wheel': [
        '0.29.0',  # virtualenv (py26)
        '0.31.1',  # virtualenv
    ],
    'argparse': [
        '1.4.0',  # virtualenv (py26)
    ],
}


LICENSES = {
    'setuptools': defaultdict(lambda: 'MIT'),
    'pip': {
        '8.1.2': 'MIT and Python and ASL 2.0 and BSD and ISC and (BSD or ASL 2.0)',
        '9.0.1': 'MIT and Python and ASL 2.0 and BSD and ISC and (BSD or ASL 2.0)',
        '9.0.3': 'MIT and Python and ASL 2.0 and BSD and ISC and (BSD or ASL 2.0)',
        '10.0.1': 'MIT and Python and ASL 2.0 and BSD and ISC and LGPLv2 and MPLv2.0 and (ASL 2.0 or BSD)',
    },
    'wheel': defaultdict(lambda: 'MIT'),
    'argparse': defaultdict(lambda: 'Python'),
} 

print('''Name:           python-bundled-wheels
Version:        0
Release:        XXX%{?dist}
Summary:        Wheels to be bundled with Python tools for virtual environments
# Individual packages have their licenses, there is no "main" package here
License:        Public Domain
URL:            https://pypi.org/
BuildArch:      noarch

BuildRequires:  python3-devel
BuildRequires:  python3dist(setuptools)
BuildRequires:  python3dist(pip)
BuildRequires:  python3dist(wheel)

# Use this script to generate the specfile:
Source0:        %{name}.py
''')

a = 1
prep = ''
for pkg, versions in PACKAGES.items():
    ext = ' zip' if pkg == 'setuptools' else ''
    b = 1
    for version in versions:
        print(f'Source{a}{b}:       %pypi_source {pkg} {version}{ext}')
        prep += f' -a{a}{b}'
        b += 1
    print()
    a += 1


print('''

%global wheeldir %{_datadir}/%{name}

%description
The purpose of this package is to build various Python wheels to be bundled
with Python tools for virtual environments, instead of taking prebuilt bundled
wheels from upstreams. This also allows to deduplicate the wheels and specify
the licenses better (no need to specify them in the projects that need those).


%package filesystem
Summary:        A directory for %{name}

%description filesystem
A directory for shared wheels.

''')


def get_bundled(pkg, version):
    if pkg != 'pip':
        return ''
    # Those lists are created with help of vendor_meta.sh
    # https://src.fedoraproject.org/rpms/python-pip/pull-request/8
    if version == '8.1.2':
        provides = '''
Provides:       bundled(python{0}dist(distlib)) = 0.2.3
Provides:       bundled(python{0}dist(html5lib)) = 1.0b8
Provides:       bundled(python{0}dist(six)) = 1.10.0
Provides:       bundled(python{0}dist(colorama)) = 0.3.7
Provides:       bundled(python{0}dist(requests)) = 2.10.0
Provides:       bundled(python{0}dist(CacheControl)) = 0.11.6
Provides:       bundled(python{0}dist(lockfile)) = 0.12.2
Provides:       bundled(python{0}dist(progress)) = 1.2
Provides:       bundled(python{0}dist(ipaddress)) = 1.0.16
Provides:       bundled(python{0}dist(packaging)) = 16.7
Provides:       bundled(python{0}dist(pyparsing)) = 2.1.1
Provides:       bundled(python{0}dist(retrying)) = 1.3.3
'''
    elif version in ('9.0.1', '9.0.3'):
        provides = '''
Provides:       bundled(python{0}dist(appdirs)) = 1.4.0
Provides:       bundled(python{0}dist(distlib)) = 0.2.4
Provides:       bundled(python{0}dist(distro)) = 1.0.1
Provides:       bundled(python{0}dist(html5lib)) = 1.0b10
Provides:       bundled(python{0}dist(six)) = 1.10.0
Provides:       bundled(python{0}dist(colorama)) = 0.3.7
Provides:       bundled(python{0}dist(requests)) = 2.11.1
Provides:       bundled(python{0}dist(CacheControl)) = 0.11.7
Provides:       bundled(python{0}dist(lockfile)) = 0.12.2
Provides:       bundled(python{0}dist(ordereddict)) = 1.1
Provides:       bundled(python{0}dist(progress)) = 1.2
Provides:       bundled(python{0}dist(ipaddress)) = 1.0.17
Provides:       bundled(python{0}dist(packaging)) = 16.8
Provides:       bundled(python{0}dist(pyparsing)) = 2.1.10
Provides:       bundled(python{0}dist(retrying)) = 1.3.3
Provides:       bundled(python{0}dist(webencodings)) = 0.5
'''
    elif version== '10.0.1':
        provides = '''
Provides:       bundled(python{0}dist(appdirs)) = 1.4.3
Provides:       bundled(python{0}dist(distlib)) = 0.2.7
Provides:       bundled(python{0}dist(distro)) = 1.2.0
Provides:       bundled(python{0}dist(html5lib)) = 1.0.1
Provides:       bundled(python{0}dist(six)) = 1.11.0
Provides:       bundled(python{0}dist(colorama)) = 0.3.9
Provides:       bundled(python{0}dist(CacheControl)) = 0.12.4
Provides:       bundled(python{0}dist(msgpack-python)) = 0.5.6
Provides:       bundled(python{0}dist(lockfile)) = 0.12.2
Provides:       bundled(python{0}dist(progress)) = 1.3
Provides:       bundled(python{0}dist(ipaddress)) = 1.0.19
Provides:       bundled(python{0}dist(packaging)) = 17.1
Provides:       bundled(python{0}dist(pyparsing)) = 2.2.0
Provides:       bundled(python{0}dist(pytoml)) = 0.1.14
Provides:       bundled(python{0}dist(retrying)) = 1.3.3
Provides:       bundled(python{0}dist(requests)) = 2.18.4
Provides:       bundled(python{0}dist(chardet)) = 3.0.4
Provides:       bundled(python{0}dist(idna)) = 2.6
Provides:       bundled(python{0}dist(urllib3)) = 1.22
Provides:       bundled(python{0}dist(certifi)) = 2018.1.18
Provides:       bundled(python{0}dist(setuptools)) = 39.0.1
Provides:       bundled(python{0}dist(webencodings)) = 0.5.1
'''
    else:
        raise NotImplementedError(f'unknown pip version {version}')
    return provides.format(2) + provides.format(3)


TEMPLATE = '''
%package {pkg}-{version}
Version:        {version}
License:        {license}
Summary:        The {pkg}-{version}-py2.py3-none-any.whl wheel
URL:            https://pypi.python.org/pypi/{pkg}
Requires:       %{{name}}-filesystem
Provides:       bundled(python2dist({pkg})) = {version}
Provides:       bundled(python3dist({pkg})) = {version}
Provides:       wheel({pkg}-{version}-py2.py3-none-any.whl) = {version}
{bundled}
%description {pkg}-{version}
The {pkg}-{version}-py2.py3-none-any.whl wheel.
'''

for pkg, versions in PACKAGES.items():
    for version in versions:
        print(TEMPLATE.format(
            pkg=pkg,
            version=version,
            license=LICENSES[pkg][version],
            bundled=get_bundled(pkg, version),
        ))


print(f'%prep\n%setup -qTc {prep}\n\n')

print('%build')

for pkg, versions in PACKAGES.items():
    print(f'for VERSION in {" ".join(versions)}; do')
    print(f'  pushd {pkg}-$VERSION')
    print('    %{__python3} -m pip wheel .')
    print('  popd')
    print('done')



print('''

%install
mkdir -p %{buildroot}/%{wheeldir}
cp -p *-*/*.whl %{buildroot}/%{wheeldir}/


%files filesystem
%dir %{wheeldir}/

''')


for pkg, versions in PACKAGES.items():
    for version in versions:
        print(f'%files {pkg}-{version}')
        if f'{pkg}-{version}' != 'setuptools-21.2.1':
            print(f'%license {pkg}-{version}/LICENSE*')
        print(f'%{{wheeldir}}/{pkg}-{version}-py2.py3-none-any.whl')
        print('\n')


print('%changelog')
print('...')
